package info.textgrid.middleware;

import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * Collection of values never changing its content.
 * </p>
 * 
 * @author Maximilian Brodhun, SUB Göttingen
 */

public final class DARIAHConstants {

  /**
   * 
   */
  private DARIAHConstants() {
    //
  }

  public static final String ITEM_IDENTIFIER_PREFIX = "hdl:";
  public static final String COLLECTIONREGISTRY_PREFIX = "dariah:collection:";
  public static final String COLLECTION_MIMETYPE = "text/vnd.dariah.dhrep.collection+turtle";
  public static final List<String> TEXTGRID_REP_ADMIN_CONTACT =
      Arrays.asList("support@de.dariah.eu");

  // Rep Identification String Constants
  public static final String DARIAH_REP_BASEURL = "repository.de.dariah.eu";
  public static final String METADATA_DC_PREFIX = "rdf:RDF/rdf:Description";

  // Error String Constants
  public static final String OAI_BAD_ARGUMENT = "BadArgument";
  public static final String OAI_METADATA_FORMAT_ERROR = "FormatError";
  public static final String OAI_NO_RECORD_MATCH = "RecordMatchError";
  public static final String OAI_NO_SET_HIERARCHY = "SetHierarchyError";

  // TextGrid Metadata Fields for OAI-PMH Request (Mapping to DC)
  public static final String CREATED = "descriptiveMetadata.dc:date";
  public static final String FORMAT = "descriptiveMetadata.dc:format";
  public static final String IDENTIFIER = "administrativeMetadata.dcterms:identifier";
  public static final String DATA_CONTRIBUTOR = "descriptiveMetadata.dc:contributor";
  public static final String[] DARIAHFields =
      {"descriptiveMetadata.dc:contributor", "descriptiveMetadata.dc:coverage",
          "administrativeMetadata.dcterms:created", "administrativeMetadata.dcterms:modified",
          "descriptiveMetadata.dc:description", "descriptiveMetadata.dc:format",
          "descriptiveMetadata.dc:identifier", "descriptiveMetadata.dc:language",
          "descriptiveMetadata.dc:publisher", "descriptiveMetadata.dc:relation",
          "descriptiveMetadata.dc:rights", "descriptiveMetadata.:source",
          "descriptiveMetadata.dc:subject", "descriptiveMetadata.dc:title",
          "descriptiveMetadata.dc:type", "administrativeMetadata.dcterms:identifier"};
  public static final String[] DARIAHCollectionFields = {"metadata.dc:title"};

  // String Arrays to define which TextGrid fields belongs to the regarding DC fields
  public static final String[] CONTRIBUTOR_LIST = {"descriptiveMetadata.dc:contributor"};
  public static final String[] COVERAGE_LIST = {"descriptiveMetadata.dc:coverage"};
  public static final String[] CREATOR_LIST = {"descriptiveMetadata.dc:cretor"};
  public static final String[] DATE_LIST = {"administrativeMetadata.dcterms:modified"};
  public static final String[] DESCRIPTION_LIST = {"descriptiveMetadata.dc:description"};
  public static final String[] FORMAT_LIST = {"descriptiveMetadata.dc:format"};
  public static final String[] IDENTIFIER_LIST = {"descriptiveMetadata.dc:identifier"};
  public static final String[] LANGUAGE_LIST = {"descriptiveMetadata.dc:language"};
  public static final String[] PUBLISHER_LIST = {"descriptiveMetadata.dc:publisher"};
  public static final String[] RELATIONS_LIST = {"descriptiveMetadata.dc:relation"};
  public static final String[] RIGHTS_LIST = {"descriptiveMetadata.dc:rights"};
  public static final String[] SOURCE_LIST = {"descriptiveMetadata.dc:source"};
  public static final String[] SUBJECT_LIST = {"descriptiveMetadata.dc:subject"};
  public static final String[] TITLE_LIST = {"descriptiveMetadata.dc:title"};
  public static final String[] TYPE_LIST = {"descriptiveMetadata.dc:type"};
  public static final String[] IDENTIFIER_LIST_FIELDS =
      {"administrativeMetadata.dcterms:identifier", "administrativeMetadata.dcterms:modified",
          "descriptiveMetadata.dc:identifier", "descriptiveMetadata.dc:date",
          "descriptiveMetadata.dc:format"};
  public static final String RANGE_FIELD = "administrativeMetadata.dcterms:modified";
  public static final String MODIFIED_FIELD = "administrativeMetadata.dcterms:modified";
}
