package info.textgrid.middleware;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Properties;

//import javax.swing.plaf.synth.SynthSpinnerUI;

import org.apache.commons.logging.LogFactory;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.search.SearchScrollRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;

import info.textgrid.middleware.oaipmh.ListIdentifiersType;
import info.textgrid.middleware.oaipmh.RequestType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;
import info.textgrid.middleware.common.TextGridMimetypes;

/**
 *
 */
public abstract class IdentifierListDelivererAbstract implements IdentifierListDelivererInterface {

  protected boolean textgrid;
  protected boolean dariah;
  private boolean foundItems;
  private long resultSize;
  private int searchResponseSize;
  public String rangeField;
  
  protected static Map<String, Integer> cursorCollector = new Hashtable<String, Integer>();
  public ResumptionTokenType resTokenForResponse;
  
  private static org.apache.commons.logging.Log log = LogFactory.getLog(RecordListDelivererAbstract.class);
  
  /**
   * @param textgrid
   * @param dariah
   */
  public IdentifierListDelivererAbstract(boolean textgrid, boolean dariah) {
    this.textgrid = textgrid;
    this.dariah = dariah;
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.IdentifierListDelivererInterface#processIdentifierList(java.lang.
   * String, java.lang.String, java.lang.String, java.lang.String)
   */
  @Override
  public ListIdentifiersType processIdentifierList(String from, String until, String set,
      String resumptionToken) throws IOException, ParseException {
    return null;
  }

  
  /**
   * @param from
   * @param until
   * @param set
   * @param resumptionToken
   * @return
   * @throws IOException
   */
  public List<String> getUriListOfAllEditions(String from, String until, String set, String resumptionToken) throws IOException {
  	
     /* InputStream input = null;
		try {
			input = new FileInputStream("/etc/dhrep/oaipmh/oaipmh.properties");
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

      Properties prop = new Properties();
      prop.load(input);*/	
		
      QueryBuilder query;
      
      System.out.println("FROM: " + from);
      System.out.println("TO: " + until);
      
      QueryBuilder rangeQuery = QueryBuilders.rangeQuery("created").from(from).to(until);
      QueryBuilder filterSandBox = QueryBuilders.matchPhraseQuery("nearlyPublished", "true");
    	if(set!=null) {
    		String[] setParts = set.split(":");
    		
          String queryField = "";
          String valueField = "";
    		
          if (setParts[0].equals(TGConstants.SET_FIELD_FOR_TEXTGRID)) {
              queryField = TGConstants.PROJECT_ID_FOR_TEXTGRID;          
              valueField = setParts[1];
          }
          System.out.println("");
          QueryBuilder matchQuery = QueryBuilders.matchPhraseQuery(queryField, valueField);
          QueryBuilder boolQuery = QueryBuilders.boolQuery().must(rangeQuery).must(matchQuery).mustNot(filterSandBox);
          
          query = boolQuery;
      }else {
    	  query = QueryBuilders.boolQuery().must(rangeQuery).mustNot(filterSandBox);
      }  	
    	
    	return getContentFromESIndex(query, resumptionToken, set);
    }
  
  /**
   * @param query
   * @param resumptionToken
   * @param set
   * @return
   * @throws IOException
   */
  private List<String> getContentFromESIndex(QueryBuilder query, String resumptionToken,
	      String set) throws IOException {
  	
	  List<String> uriList = new ArrayList<String>();
	  QueryBuilder recordFilter = QueryBuilders.boolQuery().must(query)
	          .must(QueryBuilders.matchPhraseQuery(TGConstants.FORMAT, TextGridMimetypes.EDITION));
	  
	  SearchRequest searchRequest = new SearchRequest(OAI_ESClient.getEsIndex());
	  SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
      
	  /*  InputStream input = null;
		try {
			input = new FileInputStream("/etc/dhrep/oaipmh/oaipmh.properties");
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

      Properties prop = new Properties();
      prop.load(input);*/
      
	  searchSourceBuilder.query(recordFilter);
	  searchSourceBuilder.size(100);
	  searchRequest.source(searchSourceBuilder);
	  
	  SearchResponse scrollResp = new SearchResponse();
	  
	  if (resumptionToken != null) {
		  SearchScrollRequest scrollRequest = new SearchScrollRequest(resumptionToken);
	      scrollRequest.scroll(TimeValue.timeValueHours(24L));
	      
	  	try {
	        scrollResp = OAI_ESClient.getEsClient().scroll(scrollRequest, RequestOptions.DEFAULT);
	      } catch (IOException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	      }
	  } else {
	      searchRequest.source(searchSourceBuilder);
	      searchRequest.scroll(TimeValue.timeValueHours(24L));
	      try {	    	  
	        scrollResp = OAI_ESClient.getEsClient().search(searchRequest,RequestOptions.DEFAULT);
	      } catch (IOException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	      }
	    }
	  
	  String scrollID = scrollResp.getScrollId(); 
	  long completeListSize = scrollResp.getHits().totalHits;
	  setResultSize(completeListSize);
	  
	  if (completeListSize > 0) {
		  
		  setFoundItems(true);
		  int i=0;
		  
		  for (SearchHit hit : scrollResp.getHits().getHits()) {
			  i++;
			  if (hit != null && hit.getFields() != null) {
				  uriList.add(hit.getSourceAsMap().get(TGConstants.URI).toString());				  
			  }
		  }
		  
			 if(resumptionToken!=null && resTokenForResponse.getCursor().intValue()>=completeListSize) {
				 try{
					 cursorCollector.remove(resumptionToken);
				 }catch(NullPointerException couldNotRemove) {
					 log.info("Could net remove hash value: " + resumptionToken + " from hash map");
				 }			
				resTokenForResponse.setValue("");			
			 }else {
				 
			     resTokenForResponse = OAIPMHUtilities.getResumptionToken(
			              completeListSize, resumptionToken, cursorCollector, scrollID, this.searchResponseSize, i);
			 }

	  }else {
		  setFoundItems(false);
	  }
	  
	  
	  return uriList;
  
  }
  
  
  /**
   * @param request
   * @return
   */
  public static ErrorHandler requestChecker(RequestType request) {

    ErrorHandler result = new ErrorHandler();

    // Check if metadata prefix is existing and valid.
    if (request.getMetadataPrefix() != null
        && !request.getMetadataPrefix().equals(OAIPMHUtilities.OAIDC_PREFIX)
        && !request.getMetadataPrefix().equals(TGConstants.METADATA_IDIOM_PREFIX)
        && !request.getMetadataPrefix().equals(TGConstants.METADATA_OPENAIRE_PREFIX)) {
      result.setError(TGConstants.OAI_METADATA_FORMAT_ERROR,
          "The value of the metadataPrefix: " + request.getMetadataPrefix()
              + " is not supported by the item identified by the value of: "
              + request.getIdentifier());
    }

    // Check if resumptionToken is invalid or existing.
    if (request.getResumptionToken() != null) {
      boolean restokDCExisting = IdentifierListDelivererDC.cursorCollector != null
              && IdentifierListDelivererDC.cursorCollector.containsKey(request.getResumptionToken());    		  
      boolean restokIDIOMExisting = IdentifierListDelivererIDIOM.cursorCollector != null
              && IdentifierListDelivererDC.cursorCollector.containsKey(request.getResumptionToken());
      boolean restok = cursorCollector != null
              && cursorCollector.containsKey(request.getResumptionToken());
      
       if (!restokDCExisting && !restokIDIOMExisting && !restok) {
    	  
    	  result.setError(TGConstants.OAI_BAD_RESUMPTION_TOKEN, "The value of the "
            + request.getResumptionToken() + " argument is invalid or expired.");
      }
    }

    List<String> errorValues = new ArrayList<String>();

    if (request.getResumptionToken() == null && request.getMetadataPrefix() == null) {
      errorValues.add("metadataPrefix");
    }
    if (request.getIdentifier() != null) {
      errorValues.add("identifier");
    }
    if (errorValues.size() > 0) {
      result.setError(TGConstants.OAI_BAD_ARGUMENT, "The request includes illegal arguments "
          + "or is missing required arguments: " + errorValues);
    }

    return result;
  }

	public boolean isFoundItems() {
		return foundItems;
	}

	public void setFoundItems(boolean foundItems) {
		this.foundItems = foundItems;
	}
	
	public long getResultSize() {
		return resultSize;
	}

	public void setResultSize(long resultSize) {
		this.resultSize = resultSize;
	}
	
	public ResumptionTokenType getResTokenForResponse() {
		return resTokenForResponse;
	}

	public void setResTokenForResponse(ResumptionTokenType resTokenForResponse) {
		this.resTokenForResponse = resTokenForResponse;
	}
  
	public String getRangeField() {
		return rangeField;
	}
	
	public void setRangeField(String rangeField) {
		this.rangeField = rangeField;
	}
  
}
