package info.textgrid.middleware;

import java.io.IOException;
import java.text.ParseException;
import javax.xml.datatype.DatatypeConfigurationException;
import org.apache.commons.logging.LogFactory;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.common.Strings;
import org.json.JSONObject;
import org.springframework.stereotype.Component;
import info.textgrid.middleware.oaipmh.GetRecordType;
import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.RecordType;

/**
 * FIXME Was magst Du denn hier fixen, Herr Max?
 * 
 * FIXME Habe hier mal die Methode aus OAIPMHUtilities genutzt: getRcordByIDFromElasticSearch(),
 * sollten wir auch in anderen Klassen so machen! Der Umbau auf ES6 ist so einfacher, no?
 * 
 * @author Maximilian Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2021-05-20
 * @since 2014-02-17
 */

@Component
public class RecordDelivererDC extends RecordDelivererAbstract {

  private String[] fields;
  private String[] workFields;
  private String formatField;
  private String formatToFilter;
  private String dateOfObjectCreation;
  private String relationToFurtherMetadataObject;
  private String repositoryObjectURIPrefix;
  private String identifierField;

  /*
   * Lists for the Dublin Core elements. Lists are needed since each DC fields is possible to occur
   * several times
   */

  private String[] contributorList;
  private String[] coverageList;
  private String[] creatorList;
  private String[] dateList;
  private String[] descriptionList;
  private String[] formatList;
  private String[] identifierList;
  private String[] languageList;
  private String[] publisherList;
  private String[] relationList;
  // FIXME Why is this commented out? I just adapted beans.xml file.
  // private String[] relationsForWorkList;
  private String[] rightList;
  private String[] sourceList;
  private String[] subjectList;
  private String[] titleList;
  private String[] typeList;

  private static org.apache.commons.logging.Log log = LogFactory.getLog(RecordDelivererDC.class);
  private OAI_ESClient oaiEsClient;

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordDelivererDC(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordDelivererInterface#getRecordById(java.lang.String)
   */
  public GetRecordType getRecordById(String id)
      throws ParseException, DatatypeConfigurationException {

    GetRecordType getRecordType = new GetRecordType();
    RecordType record = new RecordType();
    DublinCoreBuilder dublinCoreBuilder;

    String changedId = id;
    if (id.startsWith(this.repositoryObjectURIPrefix)) {
      // FIXME Why do we need changeID? Where is it used?
      changedId = changedId.replace(this.repositoryObjectURIPrefix, "");
    }

    // Get ES response, get record for given ID.
    GetResponse esResultObject = null;
    try {
      String[] includes = this.fields;
      String[] excludes = Strings.EMPTY_ARRAY;

      esResultObject = OAIPMHUtilities.getRcordByIDFromElasticSearch(id, includes, excludes);

    } catch (IOException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
      log.error("Could not fetch the result of ElasticSearch");
    }

    if (esResultObject.isExists()) {
      String identifier;

      if (this.dariah == true) {
        dublinCoreBuilder = putContentIntoDCFieldLists(esResultObject);
        JSONObject json = new JSONObject(esResultObject.getSourceAsMap());
        identifier = OAIPMHUtilities.fieldLoader(json, this.identifierField);
        //identifier = esResultObject.getSourceAsMap().get(this.identifierField).toString();
        String dateOfCreation = "NO_DATE_SET!";
               
        if (OAIPMHUtilities.fieldLoader(json, this.dateOfObjectCreation) != null) {
          try {
            dateOfCreation = OAIPMHUtilities
                .convertDateFormat(
                		OAIPMHUtilities.fieldLoader(json, this.dateOfObjectCreation).toString())
                .toXMLFormat();
          } catch (ParseException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
          } catch (DatatypeConfigurationException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
          }
          record.setMetadata(dublinCoreBuilder.getDC());
        }
        String setSpec = OAIPMHUtilities.fieldLoader(json, this.identifierField);
        //String setSpec =
            //DublinCoreFieldLoader.fillList(esResultObject, DARIAHConstants.IDENTIFIER_LIST).get(0);
        
        setRecordHeader(dateOfCreation, identifier, record, dublinCoreBuilder, setSpec);
      }

      if (this.textgrid == true
          && esResultObject.getSourceAsMap().get(this.formatField).equals(this.formatToFilter)) {

        String workUri = DublinCoreFieldLoader
            .fillList(esResultObject, new String[] {this.relationToFurtherMetadataObject}).get(0);
        workUri = workUri.substring(this.repositoryObjectURIPrefix.length());
        dublinCoreBuilder = putContentIntoDCFieldLists(esResultObject, furtherDCElements(workUri));
        identifier = esResultObject.getSourceAsMap().get(this.identifierField).toString();
        String dateOfCreation =
            esResultObject.getSourceAsMap().get(this.dateOfObjectCreation).toString();

        String setSpec = "project:"
            + DublinCoreFieldLoader.fillList(esResultObject, TGConstants.RELATIONS_LIST).get(0);

        setRecordHeader(OAIPMHUtilities.convertDateFormat(dateOfCreation).toString(), identifier,
            record, dublinCoreBuilder, setSpec);

        record.setMetadata(dublinCoreBuilder.getDC());
      }

      getRecordType.setRecord(record);

    } else {
      return null;
    }

    return getRecordType;
  }

  // **
  // PRIVATE METHODS
  // **

  /**
   * This function builds the header element in the XML file for the OAI-PMH response
   * 
   * @param dateOfCreation
   * @param identifier
   * @param record object containing all elements for the record in the OAI-PMH response
   * @param dbc object containing the dublin core elements for the XML file
   * @return
   */
  private static RecordType setRecordHeader(String dateOfCreation, String identifier,
      RecordType record, DublinCoreBuilder dbc, String setSpec) {

    HeaderType recordHeader = new HeaderType();

    recordHeader.setIdentifier(identifier);
    recordHeader.setDatestamp(dateOfCreation);
    recordHeader.getSetSpec().add(setSpec);

    // FIXME Do add set spec here if set existing!
    // if (set != null) {
    // recordHeader.getSetSpec().add(set);
    // }

    record.setMetadata(dbc.getDC());
    record.setHeader(recordHeader);

    return record;
  }

  /**
   * @param id
   * @return
   */
  private GetResponse furtherDCElements(String id) {

    if (!id.endsWith(".0")) {
      id = id.concat(".0");
    }

    GetResponse responseWorkValues = null;
    try {
      String[] includes = this.workFields;
      String[] excludes = Strings.EMPTY_ARRAY;

      responseWorkValues = OAIPMHUtilities.getRcordByIDFromElasticSearch(id, includes, excludes);

    } catch (IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

    return responseWorkValues;
  }

  /**
   * In case of TextGrid data from two objects are necessary. The edition object and the linked work
   * object. In result for building the DC-Object both objects from the ES-Index are needed
   * 
   * @param responseWorkValues contains the edition object
   * @param relatedWorkObject contains the work object related to edition object
   * @throws DatatypeConfigurationException
   * @throws ParseException
   */
  private DublinCoreBuilder putContentIntoDCFieldLists(GetResponse responseWorkValues,
      GetResponse relatedWorkObject) throws ParseException, DatatypeConfigurationException {

    DublinCoreBuilder result = new DublinCoreBuilder();

    // Set DublinCore lists with content from elastic search results due to configuration.
    result.setContributor(
        DublinCoreFieldLoader.fillList(responseWorkValues, this.contributorList));
    result.setCoverage(DublinCoreFieldLoader.fillList(responseWorkValues, this.coverageList));
    result.setCreator(DublinCoreFieldLoader.fillList(relatedWorkObject, this.creatorList));
    result.setDate(DublinCoreFieldLoader.fillList(relatedWorkObject, this.dateList));
    result.setDescription(
        DublinCoreFieldLoader.fillList(relatedWorkObject, this.descriptionList));
    result.setFormat(DublinCoreFieldLoader.fillList(responseWorkValues, this.formatList));
    result.setIdentifier(
        DublinCoreFieldLoader.fillList(responseWorkValues, this.identifierList));
    result.setLanguage(DublinCoreFieldLoader.fillList(responseWorkValues, this.languageList));
    result.setPublisher(DublinCoreFieldLoader.fillList(responseWorkValues, this.publisherList));
    result.setRelation(
        DublinCoreFieldLoader.fillList(responseWorkValues, this.relationList));
    result.setRights(DublinCoreFieldLoader.fillList(responseWorkValues, this.rightList));
    result.setSource(DublinCoreFieldLoader.fillList(responseWorkValues, this.sourceList));
    result.setSubject(DublinCoreFieldLoader.fillList(relatedWorkObject, this.subjectList));
    result.setTitle(DublinCoreFieldLoader.fillList(responseWorkValues, this.titleList));
    result.setType(DublinCoreFieldLoader.fillList(relatedWorkObject, this.typeList));

    return result;
  }

  /**
   * This function with just one object as parameter is for DARIAH
   * 
   * @param responseWorkValues
   * @throws DatatypeConfigurationException
   * @throws ParseException
   */
  private DublinCoreBuilder putContentIntoDCFieldLists(GetResponse responseWorkValues)
      throws ParseException, DatatypeConfigurationException {

    DublinCoreBuilder result = new DublinCoreBuilder();
    
    // Set DublinCore lists with content from elastic search results due to configuration.
    result.setContributor(
        DublinCoreFieldLoader.fillList(responseWorkValues, this.contributorList));
    result.setCoverage(DublinCoreFieldLoader.fillList(responseWorkValues, this.coverageList));
    result.setCreator(DublinCoreFieldLoader.fillList(responseWorkValues, this.creatorList));
    result.setDate(DublinCoreFieldLoader.fillList(responseWorkValues, this.dateList));
    result.setDescription(
        DublinCoreFieldLoader.fillList(responseWorkValues, this.descriptionList));
    result.setFormat(DublinCoreFieldLoader.fillList(responseWorkValues, this.formatList));
    result.setIdentifier(
    	OAIPMHUtilities.fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()), this.identifierList));
    
    result.setLanguage(DublinCoreFieldLoader.fillList(responseWorkValues, this.languageList));
    result.setPublisher(DublinCoreFieldLoader.fillList(responseWorkValues, this.publisherList));
    result.setRelation(
        DublinCoreFieldLoader.fillList(responseWorkValues, this.relationList));
    result.setRights(DublinCoreFieldLoader.fillList(responseWorkValues, this.rightList));
    result.setSource(DublinCoreFieldLoader.fillList(responseWorkValues, this.sourceList));
    result.setSubject(DublinCoreFieldLoader.fillList(responseWorkValues, this.subjectList));
    result.setTitle(DublinCoreFieldLoader.fillList(responseWorkValues, this.titleList));
    result.setType(DublinCoreFieldLoader.fillList(responseWorkValues, this.typeList));

    return result;
  }

  // **
  // GETTERS AND SETTERS
  // **

  /**
   * @param formatToFilter
   */
  public void setFormatToFilter(String formatToFilter) {
    this.formatToFilter = formatToFilter;
  }

  /**
   * @param formatField
   */
  public void setFormatField(String formatField) {
    this.formatField = formatField;
  }

  /**
   * @return
   */
  public String getDateOfObjectCreation() {
    return this.dateOfObjectCreation;
  }

  /**
   * @param dateOfObjectCreation
   */
  public void setDateOfObjectCreation(String dateOfObjectCreation) {
    this.dateOfObjectCreation = dateOfObjectCreation;
  }

  /**
   * @return
   */
  public String getRelationToFurtherMetadataObject() {
    return this.relationToFurtherMetadataObject;
  }

  /**
   * @param relationToFurtherMetadataObject
   */
  public void setRelationToFurtherMetadataObject(String relationToFurtherMetadataObject) {
    this.relationToFurtherMetadataObject = relationToFurtherMetadataObject;
  }

  /**
   * @return
   */
  public String getRepositoryObjectURIPrefix() {
    return this.repositoryObjectURIPrefix;
  }

  /**
   * @param repositoryObjectURIPrefix
   */
  public void setRepositoryObjectURIPrefix(String repositoryObjectURIPrefix) {
    this.repositoryObjectURIPrefix = repositoryObjectURIPrefix;
  }

  /**
   * @return
   */
  public String getIdentifierField() {
    return this.identifierField;
  }

  /**
   * @param identifierPrefix
   */
  public void setIdentifierField(String identifierPrefix) {
    this.identifierField = identifierPrefix;
  }

  /**
   * @param contributorList
   */
  public void setContributor(String[] contributorList) {
    this.contributorList = contributorList;
  }

  /**
   * @return
   */
  public String[] getContributor() {
    return this.contributorList;
  }

  /**
   * @param coveragesList
   */
  public void setCoverage(String[] coveragesList) {
    this.coverageList = coveragesList;
  }

  /**
   * @param creatorsList
   */
  public void setCreator(String[] creatorsList) {
    this.creatorList = creatorsList;
  }

  /**
   * @param datesList
   */
  public void setDates(String[] datesList) {
    this.dateList = datesList;
  }

  /**
   * @param descriptionsList
   */
  public void setDescriptions(String[] descriptionsList) {
    this.descriptionList = descriptionsList;
  }

  /**
   * @param formatsList
   */
  public void setFormats(String[] formatsList) {
    this.formatList = formatsList;
  }

  /**
   * @param identifiersList
   */
  public void setIdentifiers(String[] identifiersList) {
    this.identifierList = identifiersList;
  }

  /**
   * @param languagesList
   */
  public void setLanguages(String[] languagesList) {
    this.languageList = languagesList;
  }

  /**
   * @param publishersList
   */
  public void setPublishers(String[] publishersList) {
    this.publisherList = publishersList;
  }

  /**
   * @param relationsList
   */
  public void setRelations(String[] relationsList) {
    this.relationList = relationsList;
  }

  // /**
  // * @param relationsForWorkList
  // */
  // public void setRelationsForWork(String[] relationsForWorkList) {
  // this.relationsForWorkList = relationsForWorkList;
  // }

  /**
   * @param rightsList
   */
  public void setRights(String[] rightsList) {
    this.rightList = rightsList;
  }

  /**
   * @param sourcesList
   */
  public void setSources(String[] sourcesList) {
    this.sourceList = sourcesList;
  }

  /**
   * @param subjectsList
   */
  public void setSubjects(String[] subjectsList) {
    this.subjectList = subjectsList;
  }

  /**
   * @param titlesList
   */
  public void setTitles(String[] titlesList) {
    this.titleList = titlesList;
  }

  /**
   * @param typesList
   */
  public void setTypes(String[] typesList) {
    this.typeList = typesList;
  }

  /**
   * @return
   */
  public OAI_ESClient getOaiEsClient() {
    return this.oaiEsClient;
  }

  /**
   * @param oaiEsClient
   */
  public void setOaiEsClient(OAI_ESClient oaiEsClient) {
    this.oaiEsClient = oaiEsClient;
  }

  /**
   * @return
   */
  public String[] getWorkFields() {
    return this.workFields;
  }

  /**
   * @param workFields
   */
  public void setWorkFields(String[] workFields) {
    this.workFields = workFields;
  }

  /**
   * @param fields
   */
  public void setFields(String[] fields) {
    this.fields = fields;
  }

  /**
   * @return
   */
  public String[] getFields() {
    return this.fields;
  }

}
