package info.textgrid.middleware;

import java.io.IOException;
import java.io.StringReader;
import java.text.ParseException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.apache.commons.logging.LogFactory;
import org.classicmayan.tools.ClassicMayanMetsMods;
import org.elasticsearch.action.get.GetRequest;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.common.Strings;
import org.elasticsearch.search.fetch.subphase.FetchSourceContext;
import org.json.JSONObject;
import org.springframework.stereotype.Component;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import info.textgrid.middleware.oaipmh.GetRecordType;
import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.MetadataType;
import info.textgrid.middleware.oaipmh.RecordType;

/**
 * @author Maximilian Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 * @since 2020-05-20
 * @version 2019-03-12
 */

@Component
public class RecordDelivererIDIOM extends RecordDelivererAbstract {

  private String dateOfObjectCreation;
  private String dateOfLastOblectModification;
  private OAI_ESClient oaiEsClient;

  RecordType record = new RecordType();
  ClassicMayanMetsMods metsmods;

  private static org.apache.commons.logging.Log log = LogFactory.getLog(RecordDelivererIDIOM.class);

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordDelivererIDIOM(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /**
   * <p>
   * ElasticSearch request in non-public index to get the datestamps for the object creation and
   * last modification date in TextGrid-Repo.
   * </p>
   * 
   * @param idInDatabase
   * @throws ParseException
   * @throws IOException
   */
  public void setDatestamps(String idInDatabase) throws ParseException, IOException {

    String changedId = idInDatabase;

    String[] includes = new String[] {TGConstants.CREATED, TGConstants.MODIFIED_FIELD};
    String[] excludes = Strings.EMPTY_ARRAY;
    FetchSourceContext fetchSourceContext = new FetchSourceContext(true, includes, excludes);

    if (idInDatabase.startsWith("textgrid:")) {
      changedId = idInDatabase.substring("textgrid:".length());
    }

    if (!idInDatabase.contains(".0")) {
      // changedId = changedId.substring(0,changedId.length() - 1) + "0";
      changedId = changedId + ".0";
    }
    
    if (changedId.contains(".1")) {
      changedId = changedId.replace(".1", "");
    }
    // <<<<<<< HEAD
    //
    //
    // System.out.println("id in elasticsearch is: " + changedId);
    // GetRequestBuilder recordById = OAI_ESClient.getOaiESClient()
    // .prepareGet()
    // .setIndex("textgrid-nonpublic")
    // .setType(OAI_ESClient.getEsType())
    // .setFields(TGConstants.CREATED, TGConstants.MODIFIED_FIELD)
    // .setId(changedId);
    //
    // GetResponse objectInDatabase = recordById.execute().actionGet();
    //
    //
    // this.dateOfObjectCreation =
    // objectInDatabase.getField(TGConstants.CREATED).getValue().toString();
    // =======

    GetRequest getRequest =
        new GetRequest(OAI_ESClient.getEsIndex(), OAI_ESClient.getEsType(), changedId)
            .fetchSourceContext(fetchSourceContext);
    GetResponse objectInDatabase = null;

    try {
      objectInDatabase = OAI_ESClient.getEsClient().get(getRequest, RequestOptions.DEFAULT);
    } catch (IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    JSONObject json = new JSONObject();
    String[] fields = {TGConstants.CREATED, TGConstants.MODIFIED_FIELD};
    json = new JSONObject(OAIPMHUtilities
        .getRcordByIDFromElasticSearch(changedId, fields, Strings.EMPTY_ARRAY).getSource());

    System.out.println("id in elasticsearch is: " + OAIPMHUtilities
        .datestampAsString(OAIPMHUtilities.fieldLoader(json, TGConstants.CREATED)));
    System.out.println(json);
    this.dateOfObjectCreation =
        OAIPMHUtilities.datestampAsString(OAIPMHUtilities.fieldLoader(json, TGConstants.CREATED));
    // objectInDatabase.getField(TGConstants.CREATED).getValue().toString();
    // >>>>>>> feature/elastic6

    this.dateOfLastOblectModification = OAIPMHUtilities
        .datestampAsString(OAIPMHUtilities.fieldLoader(json, TGConstants.MODIFIED_FIELD));
    // objectInDatabase.getField(TGConstants.MODIFIED_FIELD).getValue().toString();
  }

  /**
   * building the record XML object for the OAI-PMH response
   * 
   * @throws ParseException
   * @throws IOException
   */
  // <<<<<<< HEAD
  //
  // public GetRecordType getRecordById(String id) {
  // ClassicMayanMetsMods metsmods;
  // =======
  public GetRecordType getRecordById(String id) throws ParseException, IOException {

    // >>>>>>> feature/elastic6
    GetRecordType grt = new GetRecordType();

    setDatestamps(id);
    String textgridBaseURI_FromID;
    if (!id.startsWith("textgrid:")) {
      textgridBaseURI_FromID = "textgrid:" + id.replace(".0", "");
    } else {
      textgridBaseURI_FromID = id;
    }
    log.info("ID for generating the METS/MODS file is: " + textgridBaseURI_FromID);
    metsmods = new ClassicMayanMetsMods(
        textgridBaseURI_FromID,
        this.dateOfObjectCreation,
        this.dateOfLastOblectModification);
    // this.record.setMetadata(idiomMets(metsmods));
    this.record.setMetadata(idiomMets());
    try {
      if (!id.startsWith("textgrid:")) {
        setRecordHeader(this.dateOfObjectCreation, "textgrid:" + id);
      } else {
        setRecordHeader(this.dateOfObjectCreation, id);
      }
    } catch (ParseException e) {
      e.printStackTrace();
      log.error(e.toString());
    }

    grt.setRecord(this.record);

    return grt;
  }

  // **
  // PRIVATE METHODS
  // **

  /**
   * <p>
   * Getting the Mets/Mods object for the ClassicMayan artifact and put them into metadata element
   * of the OAI-PMH response.
   * </p>
   * 
   * @return
   */
  // <<<<<<< HEAD
  //
  // private MetadataType idiomMets(ClassicMayanMetsMods metsmods) {
  // =======
  private MetadataType idiomMets() {
    // >>>>>>> feature/elastic6

    MetadataType metadataMets = new MetadataType();
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder;

    try {
      builder = factory.newDocumentBuilder();
      Document doc = builder.parse(new InputSource(new StringReader(metsmods.getMets())));
      metadataMets.setAny(doc.getDocumentElement());
    } catch (Exception e) {
      e.printStackTrace();
    }

    return metadataMets;
  }

  /**
   * @param dateOfCreation
   * @param identifier
   * @throws ParseException
   */
  private void setRecordHeader(String dateOfCreation, String identifier) throws ParseException {

    HeaderType recordHeader = new HeaderType();
    recordHeader.setIdentifier(identifier);
    recordHeader.setDatestamp(OAIPMHUtilities.datestampAsString(dateOfCreation));
    this.record.setHeader(recordHeader);

    // recordHeader.setStatus(StatusType.DELETED);
    // return recordHeader;
  }

  /**
   * @return
   */
  public OAI_ESClient getOaiEsClient() {
    return this.oaiEsClient;
  }

  /**
   * @param oaiEsClient
   */
  public void setOaiEsClient(OAI_ESClient oaiEsClient) {
    this.oaiEsClient = oaiEsClient;
  }

}
