package info.textgrid.middleware;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.LogFactory;
import org.apache.jena.atlas.logging.Log;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.search.SearchScrollRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;

import info.textgrid.middleware.oaipmh.RequestType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;

/**
 *
 */
public abstract class RecordListDelivererAbstract implements RecordListDelivererInterface {

  private OAI_ESClient oaiEsClient;	
	
  protected boolean textgrid;
  protected boolean dariah;

  private String rangeField;
  private String formatField;
  private String formatToFilter;
  private int searchResponseSize;
  private long resultSize;
  private boolean foundItems;
  public ResumptionTokenType resTokenForResponse;
  
  private static final int LIFETIME_RES_TOKEN = 600;
  protected static Map<String, Integer> cursorCollector = new Hashtable<String, Integer>();
  
  private static org.apache.commons.logging.Log log = LogFactory.getLog(RecordListDelivererAbstract.class);
  
  /**
   * @param textgrid
   * @param dariah
   */
  public RecordListDelivererAbstract(boolean textgrid, boolean dariah) {
    this.textgrid = textgrid;
    this.dariah = dariah;
  }

  public List<String> getUriList(String from, String to, String set, String resumptionToken) {

    QueryBuilder query;
    //System.out.println("RANGEFIELD: " + "created");
    QueryBuilder rangeQuery = QueryBuilders.rangeQuery("created").from(from).to(to);
  	
    QueryBuilder filterSandBox = QueryBuilders.matchPhraseQuery("nearlyPublished", "true");
    
  	if(set!=null && !set.equals("openaire_data")) {
  		String[] setParts = set.split(":");
  		
        String queryField = "";
        String valueField = "";
  		
        if (setParts[0].equals(TGConstants.SET_FIELD_FOR_TEXTGRID)) {
            queryField = TGConstants.PROJECT_ID_FOR_TEXTGRID;          
            valueField = setParts[1];
        }
        
        QueryBuilder matchQuery = QueryBuilders.matchPhraseQuery(queryField, valueField);
        QueryBuilder boolQuery = QueryBuilders.boolQuery().must(rangeQuery).must(matchQuery).mustNot(filterSandBox);
        
        query = boolQuery;
    }else {
       //query = rangeQuery;
       query = QueryBuilders.boolQuery().must(rangeQuery).mustNot(filterSandBox);
    }  	
  	
  	return getFieldsFromESIndex(query, resumptionToken, set);
  }
  
  
  public List<String> getFieldsFromESIndex(QueryBuilder query, String resumptionToken,
	      String set) {
	  
	  List<String> uriList = new ArrayList<String>();
	  QueryBuilder recordFilter = QueryBuilders.boolQuery().must(query)
	          .must(QueryBuilders.matchPhraseQuery("format", "text/tg.edition+tg.aggregation+xml"));
	  
	  SearchRequest searchRequest = new SearchRequest(OAI_ESClient.getEsIndex());
	  SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
	    
	  
	  searchSourceBuilder.query(recordFilter);
	  searchSourceBuilder.size(100);
	  searchRequest.source(searchSourceBuilder);
	  
	  SearchResponse scrollResp = new SearchResponse();
	  
	  if (resumptionToken != null) {

		  SearchScrollRequest scrollRequest = new SearchScrollRequest(resumptionToken);
	      scrollRequest.scroll(TimeValue.timeValueHours(24L));
	      
	  	try {
	  		
	        scrollResp = OAI_ESClient.getEsClient().scroll(scrollRequest, RequestOptions.DEFAULT);
	      } catch (IOException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	      }
	  } else {
	      searchRequest.source(searchSourceBuilder);
	      searchRequest.scroll(TimeValue.timeValueHours(24L));
	      try {
	        scrollResp = OAI_ESClient.getEsClient().search(searchRequest,RequestOptions.DEFAULT);
	      } catch (IOException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	      }
	    }
	  
	  String scrollID = scrollResp.getScrollId(); 
	  long completeListSize = scrollResp.getHits().totalHits;
	  setResultSize(completeListSize);
	  
	  if (completeListSize > 0) {
		  
		  setFoundItems(true);
		  int i=0;
		  
		  for (SearchHit hit : scrollResp.getHits().getHits()) {
			  i++;
			  if (hit != null && hit.getFields() != null) {
				  uriList.add(hit.getSourceAsMap().get(TGConstants.URI).toString());				  
			  }
		  }
		 if(resumptionToken!=null && resTokenForResponse.getCursor().intValue()>=completeListSize) {
			 try{
				 cursorCollector.remove(resumptionToken);
			 }catch(NullPointerException couldNotRemove) {
				 log.info("Could net remove hash value: " + resumptionToken + " from hash map");
			 }			
			resTokenForResponse.setValue("");			
		 }else {
		     resTokenForResponse = OAIPMHUtilities.getResumptionToken(
		              completeListSize, resumptionToken, cursorCollector, scrollID, this.searchResponseSize, i);
		 }

	    // System.out.println("TADA: ");
	     //System.out.println(cursorCollector);

	  }else {
		  setFoundItems(false);
	  }
	  //System.out.println(resTokenForResponse.getValue());
	  //System.out.println(resTokenForResponse.getCursor());
	  return uriList;
  }	  

  
  /**
   * @param request
   * @return
   */
  public static ErrorHandler requestChecker(RequestType request) {

    ErrorHandler result = new ErrorHandler();
    
    // Check if metadata prefix is existing and valid.
    if (request.getMetadataPrefix() != null
        && !request.getMetadataPrefix().equals(OAIPMHUtilities.OAIDC_PREFIX)
        && !request.getMetadataPrefix().equals(TGConstants.METADATA_IDIOM_PREFIX)
        && !request.getMetadataPrefix().equals(TGConstants.METADATA_OPENAIRE_PREFIX)) {
      result.setError(TGConstants.OAI_METADATA_FORMAT_ERROR,
          "The value of the metadataPrefix: " + request.getMetadataPrefix()
              + " is not supported by the item identified by the value of: "
              + request.getIdentifier());
    }

    // Check if resumptionToken is invalid or existing.
    if (request.getResumptionToken() != null) {
        boolean restokIDIOMExisting = RecordListDelivererIDIOM.cursorCollector != null
            && RecordListDelivererIDIOM.cursorCollector.containsKey(request.getResumptionToken());
        
        boolean restokDCExisting = RecordListDelivererDC.cursorCollector != null
            && RecordListDelivererDC.cursorCollector.containsKey(request.getResumptionToken());
        boolean restok = cursorCollector != null
                && cursorCollector.containsKey(request.getResumptionToken());
        
        if (!restokDCExisting && !restokIDIOMExisting && !restok) {          
      	  result.setError(TGConstants.OAI_BAD_RESUMPTION_TOKEN, "The value of the "
              + request.getResumptionToken() + " argument is invalid or expired.");
        }
      }
    
    

    // Check params in general.
    List<String> errorValues = new ArrayList<String>();

    if (request.getResumptionToken() == null && request.getMetadataPrefix() == null) {
      errorValues.add("metadataPrefix");
    }
    if (request.getIdentifier() != null) {
      errorValues.add("identifier");
    }
   /* if(request.getFrom()!= null && (!OAIPMHUtilities.isThisDateValid(request.getFrom()) && !OAIPMHUtilities.isThisDateValidToOtherTimeStamp(request.getFrom())))   {
    	errorValues.add("from");
    }
    if(request.getUntil() != null && (!OAIPMHUtilities.isThisDateValid(request.getUntil()) && !OAIPMHUtilities.isThisDateValidToOtherTimeStamp(request.getUntil()))) {
    	errorValues.add("until");
    }
    if(request.getFrom()!= null && request.getUntil()!=null && !OAIPMHUtilities.getFormatOfDate(request.getFrom()).equals(OAIPMHUtilities.getFormatOfDate(request.getFrom()))) {
    	errorValues.add("until, from");
    }*/
    if (errorValues.size() > 0) {
      result.setError(TGConstants.OAI_BAD_ARGUMENT, "The request includes illegal arguments "
          + "or is missing required arguments: " + errorValues);
    }

    return result;
  }

  
  
  //GETTER AND SETTER
  
	public String getRangeField() {
		return rangeField;
	}
	
	public void setRangeField(String rangeField) {
		this.rangeField = rangeField;
	}
	
	
	/**
	 * @param formatToFilter
	 */
	public void setFormatToFilter(String formatToFilter) {
	  this.formatToFilter = formatToFilter;
	}
	
	/**
	 * @param formatField
	 */
	public void setFormatField(String formatField) {
	  this.formatField = formatField;
	}

	public int getSearchResponseSize() {
		return searchResponseSize;
	}

	public void setSearchResponseSize(int searchResponseSize) {
		//System.out.println("SearchResponseSize: " + searchResponseSize);
		this.searchResponseSize = searchResponseSize;
	}

	public long getResultSize() {
		return resultSize;
	}

	public void setResultSize(long resultSize) {
		this.resultSize = resultSize;
	}

	public boolean isFoundItems() {
		return foundItems;
	}

	public void setFoundItems(boolean foundItems) {
		this.foundItems = foundItems;
	}

	public OAI_ESClient getOaiEsClient() {
		return oaiEsClient;
	}

	public void setOaiEsClient(OAI_ESClient oaiEsClient) {
		this.oaiEsClient = oaiEsClient;
	}

	public ResumptionTokenType getResTokenForResponse() {
		return resTokenForResponse;
	}

	public void setResTokenForResponse(ResumptionTokenType resTokenForResponse) {
		this.resTokenForResponse = resTokenForResponse;
	}

}
