package info.textgrid.middleware;

import java.io.IOException;
import java.text.ParseException;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import org.apache.commons.logging.LogFactory;
import org.classicmayan.tools.Queries;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.search.SearchScrollRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.index.search.MultiMatchQuery.QueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;

import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.ListRecordsType;
import info.textgrid.middleware.oaipmh.MetadataType;
import info.textgrid.middleware.oaipmh.RecordType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;

/**
 *
 */
public class RecordListDelivererIDIOM extends RecordListDelivererAbstract {

  private static org.apache.commons.logging.Log log = LogFactory.getLog(OAIPMHImpl.class);
  static Map<String, Integer> cursorCollector = new Hashtable<String, Integer>();

  private OAI_ESClient oaiEsClient;
  private boolean foundItems;

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordListDelivererIDIOM(boolean textgrid, boolean dariah) {

    super(textgrid, dariah);
  }

  /**
   * @param recordList
   * @param set
   * @param headerIdentifier
   * @return
   */
  public RecordType buildRecord(ListRecordsType recordList, String set, String headerIdentifier) {

    MetadataType metadata = new MetadataType();
    RecordType record = new RecordType();

    record.setMetadata(metadata);
    recordList.getRecord().add(record);

    return null;
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordListDelivererInterface#getRecords(java.lang.String,
   * java.lang.String, java.lang.String, java.lang.String)
   */

  @Override
  public ListRecordsType getRecords(final String from, final String to, final String set,
     
      final String resumptionToken) throws ParseException, IOException {
      System.out.println("BUFF");
      ListRecordsType recordList = new ListRecordsType();
      //
      BoolQueryBuilder recordFilterForClassicMayan;
      RangeQueryBuilder rangeQuery;
      
      rangeQuery = QueryBuilders.rangeQuery("lastModified").from(from).to(to);
      
      recordFilterForClassicMayan = QueryBuilders.boolQuery().must(rangeQuery)
    		  .must(QueryBuilders.matchPhraseQuery("project.id",
    				  "TGPR-0e926f53-1aba-d415-ecf6-539edcd8a318"))
    		  .must(QueryBuilders.matchPhraseQuery("format", "text/tg.inputform+rdf+xml"))
    		  .must(QueryBuilders.matchPhraseQuery("notes", "ARTEFACT"));
      
      SearchResponse scrollResp;
      
      SearchRequest searchRequest = new SearchRequest(OAI_ESClient.getEsIndex());
      SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
      
      scrollResp = null;
      
      searchSourceBuilder.query(recordFilterForClassicMayan);
      searchSourceBuilder.size(30);
      searchRequest.source(searchSourceBuilder);
      
      
      if (resumptionToken == null) {
    	  
          SearchScrollRequest scrollRequest = new SearchScrollRequest(resumptionToken);
          scrollRequest.scroll(TimeValue.timeValueHours(24L));  
          try {
              scrollResp = OAI_ESClient.getEsClient().scroll(scrollRequest, RequestOptions.DEFAULT);
            } catch (IOException e) {
              // TODO Auto-generated catch block
              e.printStackTrace();
            }    	  
      }else {
    	  searchRequest.source(searchSourceBuilder);
    	  searchRequest.scroll(TimeValue.timeValueHours(24L));
          try {
              scrollResp = OAI_ESClient.getEsClient().search(searchRequest, RequestOptions.DEFAULT);
            } catch (IOException e) {
              // TODO Auto-generated catch block
              e.printStackTrace();
            }
      }

      String scrollID = scrollResp.getScrollId();
      
      long completeListSize = scrollResp.getHits().totalHits;
      //long completeListSize = 45;
     
      if (completeListSize > 0) {
    	  setFoundItems(true);
    	  int i = 0;
    	  
    	  for (SearchHit hit : scrollResp.getHits().getHits()) {
    		  i++;
    		  String textgridURI = hit.getFields().get("textgridUri").getValue().toString().replace(".0","");
    		  // System.out.println("Processing: " + textgridURI);
    		  RecordDelivererIDIOM idiomRecord = new RecordDelivererIDIOM(true, false);
    		  recordList.getRecord().add(idiomRecord.getRecordById(textgridURI).getRecord());
    	  }      
      // Check the need for a resumption token!
      ResumptionTokenType resTokenForResponse = OAIPMHUtilities.getResumptionToken(
       completeListSize, resumptionToken, cursorCollector, scrollID, 30, i);
	      if (resTokenForResponse != null) {
	    	  recordList.setResumptionToken(resTokenForResponse);
	      }
      }else {
    	  setFoundItems(false);
      }
      
      return recordList;      
    
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordListDelivererInterface#setHeader(java.lang.String,
   * java.lang.String)
   */
  @Override
  public HeaderType setHeader(String set, String headerIdentifier, String setSpec) {
    return null;
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordListDelivererAbstract#getOaiEsClient()
   */
  public OAI_ESClient getOaiEsClient() {
    return this.oaiEsClient;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * info.textgrid.middleware.RecordListDelivererAbstract#setOaiEsClient(info.textgrid.middleware.
   * OAI_ESClient)
   */
  public void setOaiEsClient(OAI_ESClient oaiEsClient) {
    this.oaiEsClient = oaiEsClient;
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordListDelivererAbstract#isFoundItems()
   */
  public boolean isFoundItems() {
    return this.foundItems;
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordListDelivererAbstract#setFoundItems(boolean)
   */
  public void setFoundItems(boolean foundItems) {
    this.foundItems = foundItems;
  }

}
